import greenfoot.*;

/**
 * CLASS NAME: Domain
 * AUTHOR: danpost (greenfoot.org username)
 * DATE: January 28, 2014
 * 
 * A click in this world will produce a twister at that location.
 * The filenames of several image files are listed whose tiled image will be used by
 * the starter Twister object to cut its image out of.
 */
public class Domain extends World
{
     // image files for Twister objects
    String[] imageFilenames = { "board.jpg", "pelican.png", "tux.png", "ambulance.png", "skull.png", null };
    String baseImage; // current image
    int fileNum; // determines current image

    /**
     * This constructor is used to create a world where twisters are produced at clicked locations.
     * It creates the initial image for the Twister objects to use; also, the background is set to black 
     * and grid lines are drawn on it.
     */
    public Domain()
    {
        super(800, 600, 1);
        createBackgroundImage();
    }
    
    /**
     * This method sets a new background image to the world based on the dimensions of the image referenced by the 'fileNum' index
     */
    private void createBackgroundImage()
    {
        // use black to fill the background image
        GreenfootImage bg = getBackground();
        bg.setColor(Color.BLACK);
        bg.fill();
        // create initial image for new Twister objects        
        baseImage = imageFilenames[fileNum]; // save filename of current image
        if (baseImage != null)
        {
            GreenfootImage image = new GreenfootImage(baseImage); // create an instance of the current image
            // draw dark gray grid lines on the background image
            bg.setColor(Color.DARK_GRAY);
            for (int i=image.getWidth(); i<getWidth(); i+=image.getWidth()) bg.drawLine(i, 0, i, 600);
            for (int i=image.getHeight(); i<getHeight(); i+=image.getHeight()) bg.drawLine(0, i, 800, i);
            // draw the current image in the top-left corner of the world window
            bg.drawImage(image, 0, 0);
        }
    }
    
    /**
     * checks for user interaction and performs requested tasks if allowed
     */
    public void act()
    {
        // check for mouse presses and only add random-sized Twister at mouse location using base image if less than 200 actors in world
        if (Greenfoot.mousePressed(this) && numberOfObjects() < 200)
        {
            MouseInfo mouse = Greenfoot.getMouseInfo();
            int mx = mouse.getX(), my = mouse.getY();
            if (baseImage == null) addObject(new Twister(), mx, my);
            else addObject(new Twister(baseImage, mx, my, Greenfoot.getRandomNumber(10)*2+50), mx, my);
        }
        // check for key entry to remove all actors the world or change the image used by new Twister objects
        String key = Greenfoot.getKey();
        if ("x".equals(key)) removeObjects(getObjects(null));
        if ("space".equals(key))
        {
            fileNum = (fileNum+1)%imageFilenames.length;
            createBackgroundImage();
        }
    }
    
    /**
     * only used manually while scenario is stopped to set and view all actors at zero rotation
     */
    public void zeroRotations()
    {
        for (Object obj : getObjects(Twister.class)) ((Actor)obj).setRotation(0);
    }
}